﻿using System;
using System.IO;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Printing;

namespace BetterPagination
{
    public partial class MainPage : UserControl
    {
        static readonly Thickness desiredMargin = new Thickness(48);
        string leftOverLine;
        StreamReader streamReader;
        FileInfo fileInfo;

        public MainPage()
        {
            InitializeComponent();
        }

        void OnLoadButtonClick(object sender, RoutedEventArgs args)
        {
            OpenFileDialog dlg = new OpenFileDialog();

            if (dlg.ShowDialog().Value == true)
            {
                fileInfo = dlg.File;
                printButton.IsEnabled = true;
            }
        }

        void OnPrintButtonClick(object sender, RoutedEventArgs args)
        {
            PrintDocument printDocument = new PrintDocument();
            printDocument.BeginPrint += (object printDoc, BeginPrintEventArgs beginArgs) =>
            {
                streamReader = fileInfo.OpenText();
            };
            printDocument.EndPrint += (object printDoc, EndPrintEventArgs endArgs) =>
            {
                streamReader.Dispose();
            };
            printDocument.PrintPage += OnPrintDocumentPrintPage;
            printDocument.Print(fileInfo.Name);
        }

        void OnPrintDocumentPrintPage(object sender, PrintPageEventArgs args)
        {
            Border border = new Border
            {
                Padding = new Thickness(
                    Math.Max(0, desiredMargin.Left - args.PageMargins.Left),
                    Math.Max(0, desiredMargin.Top - args.PageMargins.Top),
                    Math.Max(0, desiredMargin.Right - args.PageMargins.Right),
                    Math.Max(0, desiredMargin.Bottom - args.PageMargins.Bottom))
            };

            StackPanel stkpnl = new StackPanel();
            border.Child = stkpnl;
            string line = leftOverLine;

            while ((leftOverLine != null) || ((line = streamReader.ReadLine()) != null))
            {
                leftOverLine = null;

                // Check for blank lines; print them with a space
                if (line.Length == 0)
                    line = " ";

                TextBlock txtblk = new TextBlock
                {
                    Text = line,
                    TextWrapping = TextWrapping.Wrap
                };

                stkpnl.Children.Add(txtblk);
                border.Measure(new Size(args.PrintableArea.Width, Double.PositiveInfinity));

                // Check if the page is now too tall
                if (border.DesiredSize.Height > args.PrintableArea.Height)
                {
                    // If so, remove some text and save it for later
                    leftOverLine = RemoveText(border, txtblk, args.PrintableArea);
                    break;
                }
            }

            if (leftOverLine == null)
                leftOverLine = streamReader.ReadLine();

            args.PageVisual = border;
            args.HasMorePages = leftOverLine != null;
        }

        string RemoveText(Border border, TextBlock txtblk, Size printableArea)
        {
            StringBuilder leftOverText = new StringBuilder();

            do
            {
                int index = txtblk.Text.LastIndexOf(' ');

                if (index == -1)
                    index = 0;

                leftOverText.Insert(0, txtblk.Text.Substring(index));
                txtblk.Text = txtblk.Text.Substring(0, index);
                border.Measure(new Size(printableArea.Width, Double.PositiveInfinity));

                if (index == 0)
                    break;
            }
            while (border.DesiredSize.Height > printableArea.Height);

            return leftOverText.ToString().TrimStart(' ');
        }
    }
}
