﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using Microsoft.Phone.Controls;

namespace EmmaReader
{
    public partial class MainPage : PhoneApplicationPage
    {
        static readonly Brush blackBrush = new SolidColorBrush(Colors.Black);
        List<String> paragraphs = new List<string>();
        BookInfo bookInfo = (Application.Current as App).BookInfoSettings;

        public MainPage()
        {
            InitializeComponent();
            PreprocessBook("Books/158.txt");
            Loaded += OnMainPageLoaded;
        }

        void PreprocessBook(string resourceName)
        {
            using (Stream stream = Application.GetResourceStream(new Uri(resourceName, UriKind.Relative)).Stream)
            {
                using (StreamReader reader = new StreamReader(stream))
                {
                    string line;
                    StringBuilder paragraph = new StringBuilder();

                    while (null != (line = reader.ReadLine()))
                    {
                        // Check for a possible paragraph termination
                        if (line.Length == 0 || line[0] == ' ')
                        {
                            if (paragraph.Length > 0)
                            {
                                paragraphs.Add(paragraph.ToString());
                                paragraph.Remove(0, paragraph.Length);
                            }

                            // Add the line anyway
                            paragraphs.Add(line);
                        }
                        else
                        {
                            // Insert a space between appended lines assembled into paragraph
                            if (paragraph.Length > 0)
                                paragraph.Append(" ");

                            paragraph.Append(line);
                        }
                    }

                    // Add the last paragraph if there's one in progress
                    if (paragraph.Length > 0)
                        paragraphs.Add(paragraph.ToString());
                }
            }
        }

        void OnMainPageLoaded(object sender, RoutedEventArgs args)
        {
            GoToPage(bookInfo.PageNumber);
        }

        void OnGestureListenerTap(object sender, GestureEventArgs args)
        {
            NextPage();
        }

        void OnGestureListenerFlick(object sender, FlickGestureEventArgs args)
        {
            if (args.Angle < 45 || args.Angle > 315)
                PreviousPage();

            else if (args.Angle > 135 && args.Angle < 225)
                NextPage();
        }

        void NextPage()
        {
            // Check for end of book
            if (bookInfo.PageNumber + 2 == bookInfo.Pages.Count &&
                bookInfo.Pages[bookInfo.PageNumber + 1].ParagraphIndex == -1)
            {
                return;
            }

            GoToPage(bookInfo.PageNumber + 1);
        }

        void PreviousPage()
        {
            if (bookInfo.PageNumber > 0)
            {
                GoToPage(bookInfo.PageNumber - 1);
            }
        }

        void GoToPage(int pageNumber)
        {
            pageNumber = Math.Max(0, pageNumber);

            // Paginate all pages up to the destination page
            while (bookInfo.Pages.Count <= pageNumber &&
                   bookInfo.Pages[bookInfo.Pages.Count - 1].ParagraphIndex != -1)
            {
                PageInfo pageInfo = bookInfo.Pages[bookInfo.Pages.Count - 1];
                int paragraphIndex = pageInfo.ParagraphIndex;
                int characterIndex = pageInfo.CharacterIndex;

                Paginate(ref paragraphIndex, ref characterIndex);
                pageInfo = new PageInfo
                {
                    ParagraphIndex = paragraphIndex,
                    CharacterIndex = characterIndex
                };
                bookInfo.Pages.Add(pageInfo);
            }

            // Check that the page number is still within bounds
            if (pageNumber > bookInfo.Pages.Count - 1 ||
                bookInfo.Pages[pageNumber].ParagraphIndex == -1)
            {
                pageNumber = bookInfo.Pages.Count - 2;
            }

            // Now paginate the destination page
            PageInfo pageInfo2 = bookInfo.Pages[pageNumber];
            int paragraphIndex2 = pageInfo2.ParagraphIndex;
            int characterIndex2 = pageInfo2.CharacterIndex;
            Paginate(ref paragraphIndex2, ref characterIndex2);

            // Add PageInfo for next page if it's not already there 
            if (pageNumber == bookInfo.Pages.Count - 1)
            {
                pageInfo2 = new PageInfo
                {
                    ParagraphIndex = paragraphIndex2,
                    CharacterIndex = characterIndex2
                };
                bookInfo.Pages.Add(pageInfo2);
            }
            bookInfo.PageNumber = pageNumber;
            pageNumberRun.Text = (bookInfo.PageNumber + 1).ToString();
        }

        void Paginate(ref int paragraphIndex, ref int characterIndex)
        {
            stackPanel.Children.Clear();

            while (paragraphIndex < paragraphs.Count)
            {
                // Skip if a blank line is the first paragraph on a page
                if (stackPanel.Children.Count == 0 &&
                    characterIndex == 0 &&
                    paragraphs[paragraphIndex].Length == 0)
                {
                    paragraphIndex++;
                    continue;
                }

                TextBlock txtblk = new TextBlock
                {
                    Text = paragraphs[paragraphIndex].Substring(characterIndex),
                    TextWrapping = TextWrapping.Wrap,
                    Foreground = blackBrush
                };

                // Check for a blank line between paragraphs
                if (txtblk.Text.Length == 0)
                    txtblk.Text = " ";

                stackPanel.Children.Add(txtblk);
                stackPanel.Measure(new Size(pageHost.ActualWidth, Double.PositiveInfinity));

                // Check if the StackPanel fits in the available height
                if (stackPanel.DesiredSize.Height > pageHost.ActualHeight)
                {
                    // Strip words off the end until it fits
                    do
                    {
                        int index = txtblk.Text.LastIndexOf(' ');

                        if (index == -1)
                            index = 0;

                        txtblk.Text = txtblk.Text.Substring(0, index);
                        stackPanel.Measure(new Size(pageHost.ActualWidth, Double.PositiveInfinity));

                        if (index == 0)
                            break;
                    }
                    while (stackPanel.DesiredSize.Height > pageHost.ActualHeight);

                    characterIndex += txtblk.Text.Length;

                    // Skip over the space
                    if (txtblk.Text.Length > 0)
                        characterIndex++;

                    break;
                }
                paragraphIndex++;
                characterIndex = 0;
            }
            // Flag the page beyond the last
            if (paragraphIndex == paragraphs.Count)
                paragraphIndex = -1;
        }
    }
}
